#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2012 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django.db import models as db


class Titled(db.Model):
    title = db.CharField(max_length=100)

    class Meta:
        abstract = True


class Dynamic(db.Model):
    duration = db.PositiveIntegerField()

    class Meta:
        abstract = True


class Video(Titled, Dynamic):
    RESOLUTION = (
            (1, '240p'), (2, '320p'), (3, '480p'),
            (4, '720p'), (5, '1080p')
    )

    author = db.CharField(max_length=100)
    resolution = db.IntegerField(choices=RESOLUTION)

    def __unicode__(self):
        return "{} - {} ({} s at {})".format(
                self.author, self.title, self.duration,
                self.get_resolution_display())

class Song(Titled, Dynamic):
    GENRE = (
            (1, 'Country'), (2, 'Folk'), (3, 'Polka'),
            (4, 'Western'), (5, 'World')
    )

    artist = db.CharField(max_length=100)
    genre = db.IntegerField(choices=GENRE)

    def __unicode__(self):
        return "{} - {} ({} s; {})".format(
                self.artist, self.title, self.duration,
                self.get_genre_display())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2011 - 2012 by Łukasz Langa
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""dj.chain
   --------

    This module provides a way to chain multiple finite iterables for
    consumption as a queryset-compatible object."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django.core.exceptions import FieldError
from lck.lang import unset


class chain(object):
    """Enables chaining multiple iterables to serve them lazily as
    a queryset-compatible object. Supports collective ``count()``,
    ``exists()``, ``exclude``, ``filter`` and ``order_by`` methods.

    Provides special overridable static methods used while yielding values:

      * ``xfilter(value)`` - yield a value only if ``xfilter(value)`` returns
                             ``True``. See known issues below.

      * ``xform(value)`` - transforms the value JIT before yielding it back.
                           It is only called for values within the specified
                           slice and those which passed ``xfilter``.

      * ``xkey(value)`` - returns a value to be used in comparison between
                          elements if sorting should be used. Individual
                          iterables should be presorted for the complete result
                          to be sorted properly.

    Known issues:

    1. If slicing or ``xfilter`` is used, reported ``len()`` is computed by
       iterating over all iterables so performance is weak. Note that ``len()``
       is used by ``list()`` when you convert your chain to a list or when
       iterating over the chain in Django templates. If this is not expected,
       you can convert to a list using a workaround like this::

           list(e for e in some_chain)

    2. Indexing on chains uses iteration underneath so performance is weak.
       This feature is only available as a last resort. Slicing on the other
       hand is also lazy."""

    def __init__(self, *iterables):
        self.iterables = iterables
        self.start = None
        self.stop = None
        self.step = None
        self.xsort = []

    @staticmethod
    def xform(value):
        """Transform the ``value`` just-in-time before yielding it back.
        Default implementation simply returns the ``value`` as is."""
        return value

    @staticmethod
    def xfilter(value=unset):
        """xfilter(value) -> bool

        Only yield the ``value`` if this method returns ``True``.
        Skip to the next iterator value otherwise. Default implementation
        always returns ``True``."""
        return True

    @staticmethod
    def xkey(value=unset):
        """xkey(value) -> comparable value

        Return a value used in comparison between elements if sorting
        should be used."""
        return value

    def copy(self, *iterables):
        """Returns a copy of this chain. If `iterables` are provided,
        they are used instead of the ones in the current object."""
        if not iterables:
            iterables = self.iterables
        result = chain(*iterables)
        result.xfilter = self.xfilter
        result.xform = self.xform
        result.xkey = self.xkey
        result.xsort = list(self.xsort)
        result.start = self.start
        result.stop = self.stop
        result.step = self.step
        return result

    def _filtered_next(self, iterator):
        """Raises StopIteration just like regular iterator.next()."""
        result = iterator.next()
        while not self.xfilter(result):
            result = iterator.next()
        return result

    def __iter__(self):
        if self.ordered:
            def _gen():
                candidates = {}
                for iterable in self.iterables:
                    iterator = iter(iterable)
                    try:
                        candidates[iterator] = [self._filtered_next(iterator),
                                                iterator]
                    except StopIteration:
                        continue
                while candidates:
                    clist = candidates.values()
                    for rule in self.xsort[::-1]:
                        reverse = rule[0] == '-'
                        if reverse:
                            rule = rule[1:]
                        clist.sort(key=lambda x: getattr(x[0], rule),
                                   reverse=reverse)
                    try:
                        to_yield, iterator = min(clist,
                                                 key=lambda x: self.xkey(x[0]))
                        yield to_yield
                    except ValueError:
                        # sequence empty
                        break
                    try:
                        candidates[iterator] = [self._filtered_next(iterator),
                                                iterator]
                    except StopIteration:
                        del candidates[iterator]
        else:
            def _gen():  # noqa
                for it in self.iterables:
                    for element in it:
                        if not self.xfilter(element):
                            continue
                        yield element
        for index, element in enumerate(_gen()):
            if self.start and index < self.start:
                continue
            if self.step and (index - (self.start or 0)) % self.step:
                continue
            if self.stop and index >= self.stop:
                break
            yield self.xform(element)

    def __getitem__(self, key):
        if isinstance(key, slice):
            if any((key.start and key.start < 0,
                    key.stop and key.stop < 0,
                    key.step and key.step < 0)):
                raise ValueError("chains do not support negative indexing")
            result = self.copy()
            result.start = key.start
            result.stop = key.stop
            result.step = key.step
        elif isinstance(key, int):
            if key < 0:
                raise ValueError("chains do not support negative indexing")
            self_without_transform = self.copy()
            self_without_transform.xform = lambda x: x
            for index, elem in enumerate(self_without_transform):
                if index == key:
                    return self.xform(elem)
            raise IndexError("chain index out of range")
        else:
            raise ValueError("chain supports only integer indexing and "
                             "slices.")
        return result

    def __len_parts__(self):
        for iterable in self.iterables:
            try:
                yield iterable.count()
            except:
                try:
                    yield len(iterable)
                except TypeError:
                    yield len(list(iterable))

    def __len__(self):
        try:
            if all((self.xfilter(),
                    not self.start,
                    not self.stop,
                    not self.step or self.step == 1)):
                # fast __len__
                sum = 0
                for sub in self.__len_parts__():
                    sum += sub
                return sum
        except TypeError:
            pass
        # slow __len__ if xfilter or slicing was used
        length = 0
        for length, _ in enumerate(self):
            pass
        return length + 1

    def _django_factory(self, _method, *args, **kwargs):
        new_iterables = []
        for it in self.iterables:
            try:
                new_iterables.append(getattr(it, _method)(*args, **kwargs))
            except (AttributeError, ValueError, FieldError):
                new_iterables.append(it)
        return self.copy(*new_iterables)

    def all(self):
        return self

    def count(self):
        """Queryset-compatible ``count`` method. Supports multiple iterables.
        """
        return len(self)

    def exclude(self, *args, **kwargs):
        """Queryset-compatible ``filter`` method. Will silently skip filtering
        for incompatible iterables."""
        return self._django_factory('exclude', *args, **kwargs)

    def exists(self):
        """Queryset-compatible ``exists`` method. Supports multiple iterables.
        """
        return bool(len(self))

    def filter(self, *args, **kwargs):
        """Queryset-compatible ``filter`` method. Will silently skip filtering
        for incompatible iterables."""
        return self._django_factory('filter', *args, **kwargs)

    def none(self, *args, **kwargs):
        return chain()

    def order_by(self, *args, **kwargs):
        """Queryset-compatible ``order_by`` method. Also supports iterables
        other than querysets but they need to be presorted for the chain to
        return consistently ordered results."""
        result = self._django_factory('order_by', *args, **kwargs)
        result.xsort.extend(args)
        try:
            if self.xkey() is unset:
                result.xkey = lambda v: 0
        except TypeError:
            pass
        return result

    @property
    def ordered(self):
        try:
            return len(self.xsort) or self.xkey() is not unset
        except TypeError:
            return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2012 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import os
import sys
from setuptools import setup, find_packages

reload(sys)
sys.setdefaultencoding('utf8')

ld_file = open(os.path.join(os.path.dirname(__file__), 'README.rst'))
try:
    long_description = ld_file.read()
finally:
    ld_file.close()
# We let it die a horrible tracebacking death if reading the file fails.
# We couldn't sensibly recover anyway: we need the long description.

setup (
    name = 'dj.chain',
    version = '0.9.0',
    author = 'Łukasz Langa',
    author_email = 'lukasz@langa.pl',
    description = "An object that enables chaining multiple iterables to serve"
        " them lazily as a queryset-compatible object.",
    long_description = long_description,
    url = 'https://github.com/ambv/dj.chain/',
    keywords = 'django dj extra contrib chain iterator iteration queryset lazy',
    platforms = ['any'],
    license = 'MIT',
    package_dir = {'': 'src'},
    packages = find_packages('src'),
    include_package_data = True,
    zip_safe = False, # if only because of the readme file
    namespace_packages = ['dj'],
    install_requires = [
        'django',
        'null==0.5',
    ],

    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 2 :: Only',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ]
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2012 by Łukasz Langa
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""Tests for chain."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django.test import TestCase


class SimpleTest(TestCase):
    def test_dummy(self):
        """Just see if the import works as expected."""
        from dj import chain  # noqa

    def test_chain_simple(self, c=None):
        from dj.chain import chain
        if not c:
            c = chain((1, 2), [3, 4], "56")
            c.xform = lambda v: int(v)
            c.xfilter = lambda v: int(v) % 2 == 0
        self.assertEqual((2, 4, 6), tuple(c))
        self.assertEqual(3, len(c))
        self.assertEqual((4, 6), tuple(c[1:]))
        self.assertEqual(2, len(c[1:]))
        self.assertEqual((4, 6), tuple(c[1:3]))
        self.assertEqual(2, len(c[1:3]))
        self.assertEqual((2, 4), tuple(c[:2]))
        self.assertEqual(2, len(c[:2]))
        self.assertEqual((2,), tuple(c[:2:2]))
        self.assertEqual(1, len(c[:2:2]))
        self.assertEqual((2, 6), tuple(c[::2]))
        self.assertEqual(2, len(c[::2]))
        self.assertEqual(6, c[2])

        try:
            c[3]
            self.fail("Index error not raised.")
        except IndexError:
            pass
        c.xfilter = lambda v: True
        self.assertEqual((1, 2, 3, 4, 5, 6), tuple(c))
        self.assertEqual(6, len(c))
        self.assertEqual((5, 6), tuple(c[4:]))
        self.assertEqual(2, len(c[4:]))
        self.assertEqual((3, 4), tuple(c[2:4]))
        self.assertEqual(2, len(c[2:4]))
        self.assertEqual((1, 2, 3, 4), tuple(c[:4]))
        self.assertEqual(4, len(c[:4]))
        self.assertEqual((1, 3), tuple(c[:4:2]))
        self.assertEqual(2, len(c[:4:2]))
        self.assertEqual((1, 4), tuple(c[::3]))
        self.assertEqual(2, len(c[::3]))
        self.assertEqual(6, c[5])
        try:
            c[6]
            self.fail("Index error not raised.")
        except IndexError:
            pass
        try:
            c[-1]
            self.fail("Value error not raised.")
        except ValueError:
            pass
        try:
            c["boo"]
            self.fail("Value error not raised.")
        except ValueError:
            pass

    def test_chain_simple_copy(self):
        from dj.chain import chain
        c = chain((1, 2), [3, 4], "56")
        c.xform = lambda v: int(v)
        c.xfilter = lambda v: int(v) % 2 == 0
        c2 = c.copy(*c.iterables)
        self.test_chain_simple(c2)

    def test_chain_sorted(self, c=None):
        from dj.chain import chain
        if not c:
            c = chain((1, 2), [3, 4], "56")
            c.xform = lambda v: int(v)
            c.xkey = lambda v: -int(v)
            c.xfilter = lambda v: int(v) % 2 == 0
        self.assertEqual((6, 4, 2), tuple(c))
        self.assertEqual(3, len(c))
        self.assertEqual((4, 2), tuple(c[1:]))
        self.assertEqual(2, len(c[1:]))
        self.assertEqual((4, 2), tuple(c[1:3]))
        self.assertEqual(2, len(c[1:3]))
        self.assertEqual((6, 4), tuple(c[:2]))
        self.assertEqual(2, len(c[:2]))
        self.assertEqual((6,), tuple(c[:2:2]))
        self.assertEqual(1, len(c[:2:2]))
        self.assertEqual((6, 2), tuple(c[::2]))
        self.assertEqual(2, len(c[::2]))
        self.assertEqual(2, c[2])
        try:
            c[3]
            self.fail("Index error not raised.")
        except IndexError:
            pass
        c.xfilter = lambda v: True
        self.assertEqual((5, 6, 3, 4, 1, 2), tuple(c))
        self.assertEqual(6, len(c))
        self.assertEqual((1, 2), tuple(c[4:]))
        self.assertEqual(2, len(c[4:]))
        self.assertEqual((3, 4), tuple(c[2:4]))
        self.assertEqual(2, len(c[2:4]))
        self.assertEqual((5, 6, 3, 4), tuple(c[:4]))
        self.assertEqual(4, len(c[:4]))
        self.assertEqual((5, 3), tuple(c[:4:2]))
        self.assertEqual(2, len(c[:4:2]))
        self.assertEqual((5, 4), tuple(c[::3]))
        self.assertEqual(2, len(c[::3]))
        self.assertEqual(2, c[5])
        try:
            c[6]
            self.fail("Index error not raised.")
        except IndexError:
            pass
        try:
            c[-1]
            self.fail("Value error not raised.")
        except ValueError:
            pass
        try:
            c["boo"]
            self.fail("Value error not raised.")
        except ValueError:
            pass

    def test_chain_sorted_copy(self):
        from dj.chain import chain
        c = chain((1, 2), [3, 4], "56")
        c.xform = lambda v: int(v)
        c.xkey = lambda v: -int(v)
        c.xfilter = lambda v: int(v) % 2 == 0
        c2 = c.copy(*c.iterables)
        self.test_chain_sorted(c2)

    def test_chain_sorted_django_factory(self):
        from dj.chain import chain
        c = chain(("8", 1, 2, "8"), [8, 3, 4, 8], "8568")
        c.xform = lambda v: int(v)
        c.xkey = lambda v: -int(v)
        c.xfilter = lambda v: int(v) % 2 == 0
        c2 = c._django_factory("__getslice__", 1, 3)
        self.test_chain_sorted(c2)


class MediaTest(TestCase):
    def setUp(self):
        from dj._chaintestproject.app.models import Video, Song
        v1 = Video(
            author='Psy', title='Gangnam Style', duration=253, resolution=5,
        )
        v1.save()
        v2 = Video(
            author='Justin Bieber', title='Baby', duration=225, resolution=4,
        )
        v2.save()
        v3 = Video(
            author='Lady Gaga', title='Bad Romance', duration=308,
            resolution=2
        )
        v3.save()
        v4 = Video(
            author='Shakira', title='Waka Waka', duration=211, resolution=3,
        )
        v4.save()
        s1 = Song(
            artist='Gotye feat. Kimbra', title='Somebody That I Used to Know',
            duration=244, genre=2,
        )
        s1.save()
        s2 = Song(
            artist='Coldplay', title='Clocks', duration=307, genre=3,
        )
        s2.save()
        s3 = Song(
            artist='Muse', title='Madness', duration=279, genre=1,
        )
        s3.save()
        s4 = Song(
            artist='Florence + The Machine', title='Spectrum', duration=218,
            genre=2,
        )
        s4.save()
        from collections import namedtuple
        Book = namedtuple('Book', "author title page_count")
        self.books = (
            Book(
                author='Charles Dickens', title='A Tale of Two Cities',
                page_count=869,
            ),
            Book(
                author='Miguel de Cervantes', title='Don Quixote',
                page_count=1212
            ),
        )
        self.Video = Video
        self.Song = Song
        self.Book = Book

    def test_model_consistency(self):
        self.assertEqual(self.Video.objects.count(), 4)
        self.assertEqual(self.Song.objects.count(), 4)
        self.assertEqual(len(self.books), 2)

    def test_basic_chain(self):
        from dj.chain import chain
        media = chain(self.Video.objects.all(), self.Song.objects.all())
        self.assertEqual(media.count(), 8)
        self.assertEqual(
            list(media.filter(duration__gt=250)),
            list(self.Video.objects.filter(title__in=('Gangnam Style',
                 'Bad Romance')).order_by('-title')) +
            list(self.Song.objects.filter(title__in=('Clocks',
                 'Madness')).order_by('title'))
        )
        self.assertEqual(media.filter(duration__gt=250).count(), 4)
        self.assertEqual(
            media.filter(duration__gt=250)[1],
            self.Video.objects.get(title='Bad Romance')
        )
        self.assertEqual(
            list(media[3:6]),
            [self.Video.objects.get(title='Waka Waka')] +
            list(self.Song.objects.filter(artist__in=('Gotye feat. Kimbra',
                 'Coldplay')).order_by('-artist'))
        )
        self.assertEqual(list(media[1::3]), [
            self.Video.objects.get(title='Baby'),
            self.Song.objects.get(artist='Gotye feat. Kimbra'),
            self.Song.objects.get(title='Spectrum'),
        ])
        self.assertEqual(list(media[:5:4]), [
            self.Video.objects.get(title='Gangnam Style'),
            self.Song.objects.get(artist='Gotye feat. Kimbra'),
        ])

    def test_collective_sort(self):
        from dj.chain import chain
        media = chain(self.Video.objects.all(), self.Song.objects.all())
        dur_asc = media.order_by('duration')
        self.assertEqual(dur_asc[0].duration, 211)
        self.assertEqual(dur_asc[1].duration, 218)
        self.assertEqual(dur_asc[2].duration, 225)
        self.assertEqual(dur_asc[3].duration, 244)
        self.assertEqual(dur_asc[4].duration, 253)
        self.assertEqual(dur_asc[5].duration, 279)
        self.assertEqual(dur_asc[6].duration, 307)
        self.assertEqual(dur_asc[7].duration, 308)
        dur_desc = media.order_by('-duration')
        self.assertEqual(dur_desc[7].duration, 211)
        self.assertEqual(dur_desc[6].duration, 218)
        self.assertEqual(dur_desc[5].duration, 225)
        self.assertEqual(dur_desc[4].duration, 244)
        self.assertEqual(dur_desc[3].duration, 253)
        self.assertEqual(dur_desc[2].duration, 279)
        self.assertEqual(dur_desc[1].duration, 307)
        self.assertEqual(dur_desc[0].duration, 308)
        title_asc = media.order_by('title')
        self.assertEqual(title_asc[0].title, 'Baby')
        self.assertEqual(title_asc[1].title, 'Bad Romance')
        self.assertEqual(title_asc[2].title, 'Clocks')
        self.assertEqual(title_asc[3].title, 'Gangnam Style')
        self.assertEqual(title_asc[4].title, 'Madness')
        self.assertEqual(title_asc[5].title, 'Somebody That I Used to Know')
        self.assertEqual(title_asc[6].title, 'Spectrum')
        self.assertEqual(title_asc[7].title, 'Waka Waka')
        title_desc = media.order_by('-title')
        self.assertEqual(title_desc[7].title, 'Baby')
        self.assertEqual(title_desc[6].title, 'Bad Romance')
        self.assertEqual(title_desc[5].title, 'Clocks')
        self.assertEqual(title_desc[4].title, 'Gangnam Style')
        self.assertEqual(title_desc[3].title, 'Madness')
        self.assertEqual(title_desc[2].title, 'Somebody That I Used to Know')
        self.assertEqual(title_desc[1].title, 'Spectrum')
        self.assertEqual(title_desc[0].title, 'Waka Waka')

    def test_heterogenic_sort(self):
        from dj.chain import chain
        media = chain(self.Video.objects.all(), self.books)
        title_asc = media.order_by('title')
        self.assertEqual(title_asc[0].title, 'A Tale of Two Cities')
        self.assertEqual(title_asc[1].title, 'Baby')
        self.assertEqual(title_asc[2].title, 'Bad Romance')
        self.assertEqual(title_asc[3].title, 'Don Quixote')
        self.assertEqual(title_asc[4].title, 'Gangnam Style')
        self.assertEqual(title_asc[5].title, 'Waka Waka')
        # No descending test since it would have rendered incorrect results
        # anyway: non-queryset iterables have to be presorted for the result
        # to be correctly ordered.
